"""
*******************************************************************************
 Nom du projet  : THC_HAL_MTTI
 Fichier        : THC_HAL_MTTI.py
 Auteur         : MTernisien-MTTI (www.mt-ti.fr)
 Date           : 08/02/2025
 Version        : v0.1
 Description    : Crée une PIN dans HAL de LinuxCNC pour recuperer la tension d'arc convertie provenant de l'arduino

 Matériel utilisé :
 - QtPlasmaC

 Fonctionnalités :
 - Configure le port serie, créé un composant puis une PIN HAL

 Historique des modifications :
 - v0.1: Initial


*******************************************************************************
"""


import serial
import hal
import time

# Configuration du port série
port = "/dev/ttyACM0"  # Modifier si nécessaire (ex: /dev/ttyUSB0)
baudrate = 115200  # Doit correspondre à Serial.begin() sur l'Arduino

# Initialisation du composant HAL
h = hal.component("arduino_voltage")
h.newpin("voltage", hal.HAL_FLOAT, hal.HAL_OUT)  # Création d'une sortie HAL pour la tension
h.ready()  # HAL devient "ready"

print("Composant HAL 'arduino_voltage' enregistré.")

# Ouverture du port série avec gestion d'erreur
try:
    ser = serial.Serial(port, baudrate, timeout=1)
    print(f"Lecture des données sur {port}...")

    while True:
        line = ser.readline().decode('utf-8').strip()  # Lecture et décodage des données série
        if line:
            try:
                voltage = float(line)  # Conversion en nombre flottant
                h["voltage"] = voltage  # Mise à jour du signal HAL
                print(f"Tension reçue : {voltage}V")
            except ValueError:
                print(f"Données invalides : {line}")

        time.sleep(0.05)  # Pause pour éviter une surcharge CPU
except serial.SerialException:
    print(f"Erreur : Impossible d'ouvrir {port}, vérifie le branchement et le port.")
except KeyboardInterrupt:
    print("Arrêt du script.")
finally:
    if 'ser' in locals() and ser.is_open:
        ser.close()
